<?php

namespace Teknisa\Libs\Service;

use Doctrine\DBAL\Exception;
use Teknisa\Libs\Exception\CustomException;
use Teknisa\Libs\Util\ActiveDirectory;
use Teknisa\Libs\Util\InstanceProvider;
use Teknisa\Libs\Util\Messages;
use Teknisa\Libs\Util\Utilities;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\Repositories;
use Zeedhi\Framework\Session\Session;

class Login {
    /** @var Session $sessionManager */
    protected $sessionManager;
    protected $loginUrl;
    protected $loginClientSecret;

    /** @var Utilities */
    private $utilities;
    /** @var Environment */
    private $environment;
    /** @var ActiveDirectory */
    private $activeDirectory;
    /** @var Repositories\UserData */
    private $userDataRepository;

    /**
     * @param $sessionManager
     * @param Utilities $utilities
     * @param Environment $environment
     * @param ActiveDirectory $activeDirectory
     * @param Repositories\UserData $userDataRepository
     */
    public function __construct($sessionManager, Utilities $utilities, Environment $environment, ActiveDirectory $activeDirectory,
                                Repositories\UserData $userDataRepository){
        $this->sessionManager  = $sessionManager;
        $this->utilities       = $utilities;
        $this->environment     = $environment;
        $this->activeDirectory = $activeDirectory;

        $config = Utilities::getLoginConfig();
        $this->loginUrl          = $config['loginUrl'];
        $this->loginClientSecret = $config['loginClientSecret'];
        $this->userDataRepository = $userDataRepository;

        $integration = Utilities::getLoginIntegration();
        $this->libClientId     = $integration['clientId'];
        $this->libClientSecret = $integration['clientSecret'];
    }

    /**
     * @return null|string
     */
    public function getBHash() {
        return $this->sessionManager->get('CURRENT_DBNAME');
    }

    /**
     * @param $userData
     * @param $loginUserInfo
     * @param $modulesVersionError
     * @param $loginToken
     * @param $loginLang
     * @throws \Exception
     * @return array
     */
    public function restoreSession($userData, $loginUserInfo, $modulesVersionError, $loginToken, $loginLang = null) {
        if(!empty($userData)) {
            if(isset($loginUserInfo['CURRENT_SECONDS'])) {
                if (microtime(true) - $loginUserInfo['CURRENT_SECONDS'] > 60) {
                    return array();
                } else {
                    unset($loginUserInfo['CURRENT_SECONDS']);
                }
            }
            if(empty($userData['USER'])) {
                return array();
            }
            if($loginLang) {
                $auxLang   = explode('_', $loginLang);
                $loginLang = $auxLang[0] . '_' . strtoupper($auxLang[1]);
            }
            $loginUser          = Utilities::zeedhiEncrypt(json_encode($loginUserInfo));
            $user               = $userData['USER'];
            $organization       = $userData['ORGANIZATION'];
            $accessProfile      = Utilities::getArrayFromAssociativeArray($userData['ACCESS_PROFILE']);
            $otherSessionValues = isset($userData['OTHER_SESSION_VALUES']) ? $userData['OTHER_SESSION_VALUES'] : array();

            $supportOperator   = isset($loginUserInfo['IS_SUPPORT_OPERATOR']) ? $loginUserInfo['IS_SUPPORT_OPERATOR'] : false;
            $supportOperatorId = isset($user['OPER_SUP_ID']) ? $user['OPER_SUP_ID'] : false;
            $user['USER_ACL_LABELS'] = empty($accessProfile) ? [] : $this->getAclLabelsInArray($accessProfile);

            $this->initSession($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo, $loginLang, $supportOperator, $supportOperatorId);
            return $this->factorySessionData($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo,
                $modulesVersionError, $loginUser, $loginToken, $supportOperator, $supportOperatorId);
        } else {
            throw new CustomException('ERR_REQUEST_USER_DATA');
        }
    }

    /**
     * @param $user
     * @param $organization
     * @param $accessProfile
     * @param $otherSessionValues
     * @param array $loginUserInfo
     * @param $modulesVersionError
     * @param $loginUser
     * @param $loginToken
     * @return array
     */
    private function factorySessionData($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo,
                                        $modulesVersionError, $loginUser, $loginToken, $supportOperator = false, $supportOperatorId = false) {
        $this->environment->setLoginTokenInSession($loginUserInfo['TOKEN']);
        if(empty($loginToken)) {
            $loginToken = InstanceProvider::getOAuthCustom()->grantAccessToken(
                $this->libClientId,
                $this->libClientSecret,
                array('sessionId' => InstanceProvider::getSession()->getId()),
                $loginUserInfo['TOKEN']
            );
        }
        $sessionData = array(
            "USER_ID"         => $user['ID'],
            "USER_NAME"       => $user['NAME'],
            "EMAIL"           => $user['EMAIL'],
            "ORGANIZATION_ID" => $user['ORGANIZATION_ID'],

            "TRADING_NAME" => $organization['TRADING_NAME'],
            "COMPANY_NAME" => $organization['COMPANY_NAME'],

            "REDIRECT_URL"     => $loginUserInfo['REDIRECT_URL'],
            "VISIBLE_ELEMENTS" => $loginUserInfo['VISIBLE_ELEMENTS'],
            "EXPIRED_ELEMENTS" => isset($loginUserInfo['EXPIRED_ELEMENTS']) ? $loginUserInfo['EXPIRED_ELEMENTS'] : [],

            "IDUTLPERFILFILIAL"   => $this->environment->getIdUtlPerfilFilial(),
            "CDPERFILSELECIONADO" => $this->environment->getNrPerfil(),

            "LOGGED"                => true,
            "SESSION_EXPIRED"       => $this->environment->getSessionExpired(),
            "IS_PENDENT_MIGRATION"  => $this->checkPendentMigrations($user['ORGANIZATION_ID'], $loginUserInfo['REPOSITORIES_INFO']),
            "MODULES_VERSION_ERROR" => $modulesVersionError,
            "ACCESS_PROFILE"        => $accessProfile,

            "LOGIN_TOKEN"         => $loginToken,
            "LOGIN_USER"          => $loginUser,
            "IS_SUPPORT_OPERATOR" => $supportOperator,
            "SUPPORT_OPERATOR_ID" => $supportOperatorId,
            "USER_ACL_LABELS" => $user['USER_ACL_LABELS']
        );

        if(isset($loginUserInfo['FILE_SERVER_API_KEY']))
            $sessionData["FILE_SERVER_API_KEY"] = $loginUserInfo["FILE_SERVER_API_KEY"];

        foreach($otherSessionValues as $key => $otherValue) {
            $sessionData[$key] = $otherValue;
        }

        return $sessionData;
    }

    /**
     * @param $nrOrg
     * @param $repositoriesInfo
     * @return bool
     */
    private function checkPendentMigrations($nrOrg, $repositoriesInfo) {
        $isPendentMigration = false;

//        try {
//            $migrationInfo = InstanceProvider::getMigrationService()->getPendentMigrations($nrOrg, $repositoriesInfo);
//        } catch (\Exception $e) {
//            $migrationInfo = array();
//        }
//
//        if(!empty($migrationInfo) && !isset($migrationInfo['ERROR'])) {
//            foreach ($migrationInfo as $migration) {
//                if ($migration['MIG_LAST_FILE_NUMBER'] && $migration['MIG_LAST_FILE_NUMBER'] < $migration['MIG_HIGHER_FILE_NUMBER']) {
//                    $isPendentMigration = true;
//                    break;
//                }
//            }
//        }

        return $isPendentMigration;
    }

    /**
     * @param $project
     * @param $eraseHash
     * @param $hash
     */
    public function logout($project, $hash) {
        $this->sessionManager->start();
        $token = $this->environment->getLoginTokenInSession();
        $user  = $this->environment->getCurrentUserId();
        $organizationId = $this->environment->getCurrentOrganizationId();

        if($hash) {
            try {
                if(Utilities::getConcurrentAccessSaveInMongoParameter()) {
                    InstanceProvider::getMongoCache()->deleteAccess($user, $project, $hash, $organizationId);
                } else {
                    $this->userDataRepository->deleteOperatorHash($user, $project, $hash, $organizationId);
                }
            } catch(\Exception $e) {}
        }

        InstanceProvider::getOAuthCustom()->deleteToken($token);
        $this->sessionManager->destroy();
    }

    /**
     * @param $user
     * @param $organization
     * @param $accessProfile
     * @param $otherSessionValues
     * @param $loginUserInfo
     * @param $loginLang
     * @param bool $supportOperator
     * @param bool $supportOperatorId
     * @throws Exception
     */
    private function initSession($user, $organization, $accessProfile, $otherSessionValues, $loginUserInfo, $loginLang,
                                 $supportOperator = false, $supportOperatorId = false)
    {
        if(!isset($loginUserInfo['CURRENT_SECONDS'])) {
            $loginUserInfo['CURRENT_SECONDS'] = microtime(true);
        }

        $idUtlPerfilFilial = $this->userDataRepository->getIdUtlPerfilFilial($user['ORGANIZATION_ID']);
        if($idUtlPerfilFilial['IDUTLPERFILFILIAL'] == 'S') {
            $perfilSelecionado = $this->userDataRepository->getPerfilSelecionado($user['ORGANIZATION_ID'], $user['ID']);
            $this->sessionManager->set('CDPERFILSELECIONADO', $perfilSelecionado['CDPERFILSELECIONADO']);
        }

        $this->sessionManager->start();

        $this->sessionManager->set('USER_ID',         $user['ID']);
        $this->sessionManager->set('USER_NAME',       $user['NAME']);
        $this->sessionManager->set('EMAIL',           $user['EMAIL']);
        $this->sessionManager->set('ORGANIZATION_ID', $user['ORGANIZATION_ID']);
        $this->sessionManager->set('TRADING_NAME',    $organization['TRADING_NAME']);

        $this->sessionManager->set('CDOPERADOR',  $user['ID']);
        $this->sessionManager->set('NMOPERADOR',  $user['NAME']);
        $this->sessionManager->set('DSEMAILOPER', $user['EMAIL']);
        $this->sessionManager->set('NRORG',       $user['ORGANIZATION_ID']);
        $this->sessionManager->set('NMORG',       $organization['TRADING_NAME']);

        $this->sessionManager->set('IDUTLPERFILFILIAL', $idUtlPerfilFilial['IDUTLPERFILFILIAL']);

        if(isset($loginUserInfo['FILE_SERVER_API_KEY'])) {
            $this->sessionManager->set('FILE_SERVER_API_KEY', $loginUserInfo['FILE_SERVER_API_KEY']);
        }

        $this->sessionManager->set('ACCESS_PROFILE',   $accessProfile);
        $this->sessionManager->set('LOGIN_USER_INFO',  $loginUserInfo);
        $this->sessionManager->set('LOGIN_LANGUAGE',   $loginLang);

        $this->sessionManager->set('USER_ACL_LABELS', $user['USER_ACL_LABELS']);

        $this->sessionManager->set('IS_SUPPORT_OPERATOR', $supportOperator);
        $this->sessionManager->set('SUPPORT_OPERATOR_ID', $supportOperatorId);
        if ($supportOperator) {
            $this->sessionManager->set('SUPPORT_OPERATOR_EXPIRATION_DATE', $user['DTLIMACESSOSUP']);
        }

        foreach($otherSessionValues as $key => $otherValue) {
            $this->sessionManager->set($key, $otherValue);
        }
    }

    /**
     * @param $timeout
     * @return array
     */
    public function validateExpiredSession($timeout) {
        $lastRequestTime = $this->sessionManager->get('LAST_REQUEST_TIME');
        $sessionInfo = array('IS_EXPIRED' => false);

        if($lastRequestTime) {
            $date = new \DateTime();
            $actualRequestTime = $date->getTimestamp();
            $diffTime = $actualRequestTime - $lastRequestTime;

            if($diffTime >= ($timeout / 1000)) {
                $sessionInfo['IS_EXPIRED'] = true;
                $this->environment->setSessionExpired(true);
            } else {
                $sessionInfo['TIMEOUT'] = $timeout - ($diffTime * 1000);
            }
        }

        return $sessionInfo;
    }

    /**
     * @param $user
     * @param $accessProfile
     * @param $otherSessionValues
     * @param $loginUser
     * @return array
     */
    public function updateSession($user, $accessProfile, $otherSessionValues, $loginUser) {
        $loginUserInfo = json_decode(Utilities::zeedhiDecrypt($loginUser), true);

        if(!empty($user)) {
            if (isset($user['NAME'])) {
                $this->sessionManager->set('USER_NAME', $user['NAME']);
                $loginUserInfo['USER_DATA']['USER']['NAME'] = $user['NAME'];
            }
            if (isset($user['EMAIL'])) {
                $this->sessionManager->set('EMAIL', $user['EMAIL']);
                $loginUserInfo['USER_DATA']['USER']['EMAIL'] = $user['EMAIL'];
            }
        }
        if(!empty($accessProfile)) {
            $this->sessionManager->set('ACCESS_PROFILE', Utilities::getArrayFromAssociativeArray($accessProfile));
            $loginUserInfo['USER_DATA']['ACCESS_PROFILE'] = $accessProfile;
        }
        if(!empty($otherSessionValues)) {
            foreach ($otherSessionValues as $key => $otherValue) {
                $this->sessionManager->set($key, $otherValue);
                $loginUserInfo['USER_DATA']['OTHER_SESSION_VALUES'][$key] = $otherValue;
            }
        }

        return array('user' => Utilities::zeedhiEncrypt(json_encode($loginUserInfo)));
    }

    /**
     * @param $accessProfile
     * @return array
     * @throws Exception
     */
    private function getAclLabelsInArray($accessProfile)
    {
        $labelsInArray = [];
        $aclLabels = InstanceProvider::getAclLabelsService()->getLabelsByNrperfilacesso($accessProfile);
        foreach ($aclLabels as $row) {
            if (!in_array($row['LABEL'], $labelsInArray)) {
                $labelsInArray[] = $row['LABEL'];
            }
        }

        return $labelsInArray;
    }
}
