<?php
namespace Zeedhi\Tools\DatabaseReader;

abstract class AdapterReader {

    /**
     * Get name From all tables in database.
     *
     * @return string[]
     */
    public abstract function getAllTableNames();

    /**
     * Get a table DTObject with all data about chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception  When table was not found in database.
     *
     * @return DTO\Table
     */
    public abstract function getTable($tableName);

    /**
     * Get foreign key from chosen table with given name.
     *
     * @param string $tableName Name from chosen table.
     * @param string $constraintName Name of constraint.
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When table was not found in database.
     *
     * @return DTO\Constraints\ForeignKey
     */
    public abstract function getForeignKeyByNameAndTableName($constraintName, $tableName);
    public abstract function getDataFromRefConstraint($constraintName);
    protected abstract function getConsColumnsRow($constraintName);
    protected abstract function getDTOConsColumnFromRow($consColumnRow);
    protected abstract function getConstraintRow($tableName, $conType);
    protected abstract function getTableRow($tableName);
    protected abstract function getDTOUniqueConstraintsFromRows($uniqueConstraintRow);
    protected abstract function getDTOPrimaryKeyFromRow($primaryKeyRow);
    protected abstract function getColumnRows($tableName);
    protected abstract function getForeignKeyRows($tableName);
    protected abstract function getDTOColumnFromRow($columnRow);

    /**
     * @param $foreignKeyRow
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When Foreign Key's delete rule is not supported.
     *
     * @return DTO\Constraints\ForeignKey
     */
    protected abstract  function getDTOForeignKeyFromRow($foreignKeyRow);

    protected function completeConsColumnForConstraint(DTO\Constraint $constraint) {
        $consColumnRows = $this->getConsColumnsRow($constraint->getConstraintName());
        foreach($consColumnRows as $consColumnRow) {
            $constraint->setConsColumn($this->getDTOConsColumnFromRow($consColumnRow));
        }
    }

    protected abstract function getUniqueConstraintRows($tableName);

    protected function getDTOForeignKeysFromRows(array $foreignKeyRows) {
        $DTOForeignKeys = array();
        foreach($foreignKeyRows as $foreignKeyRow) {
            $DTOForeignKeys[] = $this->getDTOForeignKeyFromRow($foreignKeyRow);
        }

        return $DTOForeignKeys;
    }

    /**
     * Get unique constraints From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When table was not found in database.
     *
     * @return DTO\Constraints\Unique[]
     */
    public function getUniqueConstraintsFromTable($tableName) {
        $uniqueConstraintRows = $this->getUniqueConstraintRows($tableName);
        $DTOUniqueConstraints = $this->getDTOUniqueConstraintsFromRows($uniqueConstraintRows);
        foreach($DTOUniqueConstraints as $DTOUniqueConstraint) {
            $this->completeConsColumnForConstraint($DTOUniqueConstraint);
        }
        return $DTOUniqueConstraints;
    }

    /**
     * Get all columns From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When table was not found in database.
     *
     * @return DTO\Column[]
     */
    public function getColumnsFromTable($tableName) {
        $columnsRow = $this->getColumnRows($tableName);
        $DTOColumns = array();
        foreach($columnsRow as $columnRow) {
            $DTOColumns[$columnRow["COLUMN_NAME"]] = $this->getDTOColumnFromRow($columnRow);
        }
        return $DTOColumns;
    }

    /**
     * Get primary key From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When table was not found in database.
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When table has no primary key.
     *
     * @return DTO\Constraints\PrimaryKey
     */
    public function getPrimaryKeyFromTable($tableName) {
        $primaryKeyRow = $this->getPrimaryKeyRow($tableName);
        $DTOPrimaryKey = $this->getDTOPrimaryKeyFromRow($primaryKeyRow);
        $this->completeConsColumnForConstraint($DTOPrimaryKey);
        return $DTOPrimaryKey;
    }

    protected abstract function getPrimaryKeyRow($tableName);

    /**
     * Get foreign keys From chosen table.
     *
     * @param string $tableName Name from chosen table.
     *
     * @throws \Zeedhi\Tools\DatabaseReader\Exception When table was not found in database.
     *
     * @return DTO\Constraints\ForeignKey[]
     */
    public function getForeignKeysFromTable($tableName) {
        $foreignKeyRows = $this->getForeignKeyRows($tableName);
        $DTOForeignKeys = $this->getDTOForeignKeysFromRows($foreignKeyRows);
        foreach($DTOForeignKeys as $DTOForeignKey) {
            $this->completeConsColumnForConstraint($DTOForeignKey);
        }
        return $DTOForeignKeys;
    }

    /**
     * Get the sequence of that table.
     *
     * @param string $tableName  Name of table.
     * @param string $columnName Name of column that uses that sequence.
     *
     * @return DTO\Sequence
     */
    public abstract function getSequenceFromTableAndColumn($tableName, $columnName);

    /**
     * Get the Check Constraints for chosen Table.
     *
     * @param string $tableName The Chosen Table
     *
     * @return DTO\Constraints\Check[]
     */
    public function getCheckConstraints($tableName) {
        return array();
    }
}